#include <glib/gi18n.h>
#include <expat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "scp_log.h"
#include "scp_config.h"
#define DEF_SOPCAST_BIN "sp-sc"
#define DEF_PLAYER_BIN  "totem"
#define DEF_PLAYER_PARAMS    ""
#define DEF_CH_LIST_URI "http://channel.sopcast.com/chlist.xml"
void _MainDlgResponse(GtkDialog *dialog, gint respId, gpointer data)
{
    ScpConfig *sc;
    
    sc = (ScpConfig*)data;
    switch (respId) {
        case GTK_RESPONSE_ACCEPT:
        puts("accept");
        ScpConfigSave(sc);
        break;
        case GTK_RESPONSE_REJECT:
        puts("reject");
        gtk_widget_hide((GtkWidget*)((ScpConfig*)data)->mainDlg);
        break;
    }
}

enum {
    Elem,
    ElemConfig,
    ElemConfigChlisturi,
    ElemConfigSopcast,
    ElemConfigSopcastBin,
    ElemConfigPlayer,
    ElemConfigPlayerBin,
    ElemConfigPlayerParams        
};

void _rawstart(void *data, const char *el, const char **attr) 
{
    ScpConfig* scc;
    
    scc = (ScpConfig*)data;   
    
    if (!strcmp(el, "params")) {
        switch (scc->curElem) {
            case ElemConfigPlayer:
                scc->curElem = ElemConfigPlayerParams;
                break;
        }
    } else if (!strcmp(el, "bin")) {
        switch (scc->curElem) {
            case ElemConfigPlayer:
                scc->curElem = ElemConfigPlayerBin;
                break;
            case ElemConfigSopcast:
                scc->curElem = ElemConfigSopcastBin;
                break;
        }
    } else if (!strcmp(el, "player")) {
        scc->curElem = ElemConfigPlayer;
    } else if (!strcmp(el, "sopcast")) {
        scc->curElem = ElemConfigSopcast;
    }else if (!strcmp(el, "channels_list_uri")) {
        scc->curElem = ElemConfigChlisturi;
    } else if (!strcmp(el, "config")) {
        scc->curElem = ElemConfig;
    }
}

void _rawend(void *data, const char *el) 
{
    ScpConfig* scc;
    
    scc = (ScpConfig*)data;   
    
    if (!strcmp(el, "params")) {
        switch (scc->curElem) {
            case ElemConfigPlayerParams:
                scc->curElem = ElemConfigPlayer;
                break;
        }
    } else if (!strcmp(el, "bin")) {
        switch (scc->curElem) {
            case ElemConfigPlayerBin:
                scc->curElem = ElemConfigPlayer;
                break;
            case ElemConfigSopcastBin:
                scc->curElem = ElemConfigSopcast;
                break;
        }
    } else if (!strcmp(el, "player")) {
        scc->curElem = ElemConfig;
    } else if (!strcmp(el, "sopcast")) {
        scc->curElem = ElemConfig;
    } else if (!strcmp(el, "channels_list_uri")) {
        scc->curElem = ElemConfig;
    } else if (!strcmp(el, "config")) {
        scc->curElem = Elem;
    }    
}

void _dataHndl(void *data, const XML_Char *s, int len)
{
    ScpConfig* scc;
    
    scc = (ScpConfig*)data;
    switch (scc->curElem) {
        case ElemConfigPlayerBin:
            g_string_assign(scc->playerExe, "");
            g_string_append_len(scc->playerExe, s, len);
            break;
        case ElemConfigPlayerParams:
            g_string_assign(scc->playerParams, "");
            g_string_append_len(scc->playerParams, s, len);
            break;
        case ElemConfigSopcastBin:
            g_string_assign(scc->scExe, "");
            g_string_append_len(scc->scExe, s, len);
            break;
        case ElemConfigChlisturi:
            g_string_assign(scc->chListUri, "");
            g_string_append_len(scc->chListUri, s, len);
            break;
    }
}

static int _ScpConfigLoadConf(ScpConfig* cfg)
{
    XML_Parser ps;
    int fd;
    int ret;
    int readed;
    char buf[1024];
    
    ps = XML_ParserCreate(NULL);
    XML_SetElementHandler(ps, _rawstart, _rawend);
    XML_SetCharacterDataHandler(ps, _dataHndl);
    XML_SetUserData(ps, cfg);
    if ((ret = fd = open(cfg->confFile->str, O_RDONLY, 0644))<0) {
        SCP_ERR_OUT(ERR_FREE_PARSER, "open config file failed: %s\n", cfg->confFile->str);
    }
    if (lseek(fd, 0, SEEK_SET)) {
        SCP_ERR_OUT(ERR_CLOSE_FD, "seek failed\n");
    }
    while ((readed = (read(fd, buf, sizeof(buf)))) >0 ) {
        XML_Parse(ps, buf, readed, 0);
    }
    XML_Parse(ps, buf, 0, 1);
    if (readed != 0) {
        SCP_ERR_OUT(ERR_CLOSE_FD, "read config file failed\n");
    }
    close(fd);
    XML_ParserFree(ps);
    return 0;
ERR_CLOSE_FD:
    close(fd);
ERR_FREE_PARSER:
    XML_ParserFree(ps);
ERR_OUT:
    return ret;
}

/*
<config>
<channels_list_uri></channels_list_uri>
<sopcast>
	<bin>sp-sc</bin>
</sopcast>
<player>
	<bin>totem</bin>
	<params></params>
</player>
</config>
*/
static int _ScpConfigSaveConf(ScpConfig* cfg)
{
    int fd;
    int ret = -1;
    const char *tmp;
    
    SCP_DBG("confFile:%s\n", cfg->confFile->str);
    if ((ret = fd = open(cfg->confFile->str, O_WRONLY|O_CREAT, 0644))<0) {
        SCP_ERR_OUT(ERR_OUT, "open config file failed: %s\n", cfg->confFile->str);
    }
    SCP_DBG("\n");
    tmp = "<config><channels_list_uri>";
    write(fd, tmp, strlen(tmp));
    write(fd, cfg->chListUri->str, cfg->chListUri->len);
    tmp = "</channels_list_uri><sopcast><bin>";
    write(fd, tmp, strlen(tmp));
	write(fd, cfg->scExe->str, cfg->scExe->len);
	tmp = "</bin></sopcast><player><bin>";
    write(fd, tmp, strlen(tmp));
    write(fd, cfg->playerExe->str, cfg->playerExe->len);
    tmp = "</bin><params>";
	write(fd, tmp, strlen(tmp));
	write(fd, cfg->playerParams->str, cfg->playerParams->len);
	tmp = "</params></player></config>";
	write(fd, tmp, strlen(tmp));
    close(fd);
    return 0;
ERR_CLOSE_FD:
    close(fd);
ERR_OUT:
    return ret;
}

int ScpConfigInit(ScpConfig* cfg)
{
    GtkGrid* grid;
    const char* home;
    GtkWidget* lbl;
    
    cfg->chListUri = g_string_new("");
    cfg->scExe = g_string_new("");
	cfg->playerExe = g_string_new("");
	cfg->playerParams = g_string_new("");
    cfg->confFile = g_string_new("");
    
    if ((home=getenv("HOME"))) {
        g_string_assign(cfg->confFile, home);
    } else {
        g_string_assign(cfg->confFile, ".");
    }
    g_string_append(cfg->confFile, "/.config");
    mkdir(cfg->confFile->str, 0755);
    g_string_append(cfg->confFile, "/scplayer");
    mkdir(cfg->confFile->str, 0755);
    g_string_append(cfg->confFile, "/config.xml");
    
    if (_ScpConfigLoadConf(cfg)) {
        SCP_DBG("\n");
        g_string_assign(cfg->chListUri, DEF_CH_LIST_URI);
        g_string_assign(cfg->scExe, DEF_SOPCAST_BIN);
        g_string_assign(cfg->playerExe, DEF_PLAYER_BIN);
        g_string_assign(cfg->playerParams, DEF_PLAYER_PARAMS);
        SCP_DBG("\n");
        ScpConfigSave(cfg);
    }
    
    if (!cfg->chListUri->len)
        g_string_assign(cfg->chListUri, DEF_CH_LIST_URI);
    if (!cfg->scExe->len) 
        g_string_assign(cfg->scExe, DEF_SOPCAST_BIN);
    if (!cfg->playerExe->len) 
        g_string_assign(cfg->playerExe, DEF_PLAYER_BIN);
    if (!cfg->playerParams->len)
        g_string_assign(cfg->playerParams, DEF_PLAYER_PARAMS);
    
    SCP_DBG("sopcast:%s\nplayer:%s\nplayerParams:%s\n",
            cfg->scExe->str, cfg->playerExe->str, cfg->playerParams->str);
    cfg->mainDlg = (GtkDialog*)gtk_dialog_new_with_buttons(_("SCPlayer Settings"), NULL, 0, 
             GTK_STOCK_OK,
             GTK_RESPONSE_ACCEPT,
             GTK_STOCK_CANCEL,
             GTK_RESPONSE_REJECT,
             NULL);
    gtk_widget_set_size_request((GtkWidget*)cfg->mainDlg, 400, 200);
    g_signal_connect(cfg->mainDlg, "response", G_CALLBACK(_MainDlgResponse), cfg);
    
    grid = (GtkGrid*)gtk_grid_new();
    gtk_grid_set_row_spacing(grid, 8);
    gtk_grid_set_column_spacing(grid, 8);
    gtk_widget_set_hexpand((GtkWidget*)grid, TRUE);
    gtk_container_set_border_width((GtkContainer*)grid, 8);
    gtk_container_add((GtkContainer*)gtk_dialog_get_content_area(cfg->mainDlg), (GtkWidget*)grid);
    
    gtk_grid_attach(grid, (GtkWidget*)gtk_label_new(_("sopcast:")), 0, 0, 1, 1);
    
    cfg->scExeChsBtn = (GtkFileChooserButton*)gtk_file_chooser_button_new(_("Select sopcast binary"), GTK_FILE_CHOOSER_ACTION_OPEN);
    gtk_file_chooser_set_current_folder((GtkFileChooser*)cfg->scExeChsBtn, "/");
    gtk_grid_attach(grid, (GtkWidget*)cfg->scExeChsBtn, 1, 0, 1, 1);
    
    gtk_grid_attach(grid, (GtkWidget*)gtk_label_new(_("player:")), 0, 1, 1, 1);
    
    cfg->playerExeChsBtn = (GtkFileChooserButton*)gtk_file_chooser_button_new(_("Select sopcast binary"), GTK_FILE_CHOOSER_ACTION_OPEN);
    gtk_file_chooser_set_current_folder((GtkFileChooser*)cfg->playerExeChsBtn, "/");
    gtk_grid_attach(grid, (GtkWidget*)cfg->playerExeChsBtn, 1, 1, 1, 1);
    
    lbl = gtk_label_new(_("extra player params:"));
    gtk_grid_attach(grid, lbl, 0, 2, 1, 1);
    
    cfg->playerParamsEty = (GtkEntry*)gtk_entry_new();
    gtk_grid_attach(grid, (GtkWidget*)cfg->playerParamsEty, 1, 2, 1, 1);
    
    gtk_grid_attach(grid, (GtkWidget*)gtk_label_new(_("channel list URI:")), 0, 3, 1, 1);
    cfg->chListUriEty = (GtkEntry*)gtk_entry_new();
    gtk_widget_set_hexpand((GtkWidget*)cfg->chListUriEty, TRUE);
    gtk_grid_attach(grid, (GtkWidget*)cfg->chListUriEty, 1, 3, 1, 1);
    
    gtk_widget_show_all((GtkWidget*)grid);
    gtk_widget_hide(lbl);
    gtk_widget_hide((GtkWidget*)cfg->playerParamsEty);
    return 0;
}

int ScpConfigShowDialog(ScpConfig* cfg)
{
    /* todo: set UI form config->xx */
    gtk_file_chooser_set_filename((GtkFileChooser*)cfg->scExeChsBtn, cfg->scExe->str);
    gtk_file_chooser_set_filename((GtkFileChooser*)cfg->playerExeChsBtn, cfg->playerExe->str);
    gtk_entry_set_text(cfg->playerParamsEty, cfg->playerParams->str);
    gtk_entry_set_text(cfg->chListUriEty, cfg->chListUri->str);
	gtk_widget_show((GtkWidget*)cfg->mainDlg);
	return 0;
}

int ScpConfigSave(ScpConfig* cfg)
{
	int ret;
	gchar* uri;
	
	/* todo: set config->xxx from UI */
	
	uri = gtk_file_chooser_get_filename((GtkFileChooser*)cfg->scExeChsBtn);
	if (uri) {
		g_string_assign(cfg->scExe, uri);
		g_free(uri);
	}
	SCP_DBG("scExe set to %s\n", cfg->scExe->str);
	
	uri = gtk_file_chooser_get_filename((GtkFileChooser*)cfg->playerExeChsBtn);
	if (uri) {
		g_string_assign(cfg->playerExe, uri);
		g_free(uri);
	}
	SCP_DBG("playerExe set to %s\n", cfg->playerExe->str);
	
	g_string_assign(cfg->playerParams, gtk_entry_get_text(cfg->playerParamsEty));
	g_string_assign(cfg->chListUri, gtk_entry_get_text(cfg->chListUriEty));
	
    ret = _ScpConfigSaveConf(cfg);
    gtk_widget_hide((GtkWidget*)cfg->mainDlg);
    return ret;
}

int ScpConfigExit(ScpConfig* cfg)
{
    gtk_widget_destroy((GtkWidget*)cfg->mainDlg);
    g_string_free(cfg->confFile, TRUE);
    g_string_free(cfg->scExe, TRUE);
    g_string_free(cfg->playerExe, TRUE);
    g_string_free(cfg->playerParams, TRUE);
    g_string_free(cfg->chListUri, TRUE);
    //g_object_unref(cfg->settings);  /* why no need to free settings? */
    return 0;
}

